package alex.carmessenger;

import android.content.ContentValues;
import android.content.Context;
import android.database.Cursor;
import android.database.SQLException;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteOpenHelper;
import android.location.Location;
import android.util.Log;

import com.google.android.gms.maps.model.LatLng;
import com.google.android.gms.maps.model.Marker;

import java.io.UnsupportedEncodingException;
import java.security.InvalidAlgorithmParameterException;
import java.security.InvalidKeyException;
import java.security.NoSuchAlgorithmException;
import java.security.spec.InvalidParameterSpecException;
import java.text.DateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * Created by alex on 05/03/2017.
 * This is the Database Adapter class. It handles all the database functionality.
 * The first part of the class is declaring all the column names, database table names, and some
 * basic sql statements. This class is designed to create the database in the phone it is run on, that
 * is why there are creation sql statements within the class. Each time the application is run,
 * the application checks for a version of the database within the phone. If no database is found, then
 * it would be created and then updated with each addition from the other classes using this class.
 */
public class DBAdapter {
    static final String KEY_UNIQUEID = "uniqueID";
    static final String KEY_PASSWORD = "Password";
    static final String KEY_REGISTRATIONNUMBER = "RegistrationNumber";
    static final String KEY_SURNAME = "surname";
    static final String TAG = " DBAdapter";
    static final String KEY_ID = "_id";
    static final String KEY_MESSAGE = "Message";
    static final String KEY_DATE = "MessageDate";
    static final String KEY_REGNO = "RecipientRegNo";
    static final String KEY_LAT = "Lat";
    static final String KEY_LNG = "Lng";
    static final String KEY_STREET_NAME = "StreetName";
    static final String KEY_ACCIDENT_TIME = "AccidentTime";
    static final String[] cols = {KEY_ID,KEY_LAT, KEY_LNG, KEY_STREET_NAME, KEY_ACCIDENT_TIME};


    static final String DATABASE_NAME = "CarMessagingSystem.db";
    static final String DATABASE_TABLE = "CarIdentifiers";
    static final String DATABASE_TABLE2 = "Messages";
    static final String DATABASE_TABLE3 = "AccidentInformation";
    static final int DATABASE_VERSION = 1;

    static final String DATABASE_CREATE =
            "create table CarIdentifiers (uniqueID text primary key not null, " +
                    "Password text not null, RegistrationNumber text not null, surname text not null);";

    static final String DATABASE_CREATE2 = "create table Messages (_id integer primary key not null autoincrement,"
            + "Message text not null, MessageDate text not null, RecipientRegNo text not null";

    static final String DATABASE_CREATE3 = "create table AccidentInformation(_id integer primary key not null autoincrement," +
            "Lat text not null, Lng text not null, StreetName text not null, accidentTime DATETIME not null)";

    final Context context;
    DatabaseHelper DBHelper;
    public SQLiteDatabase db;

    public DBAdapter(Context ctx)
    {
        this.context = ctx;
        DBHelper = new DatabaseHelper(context);
    }

    private static class DatabaseHelper extends SQLiteOpenHelper
    {
        DatabaseHelper(Context context)
        {
            super(context, DATABASE_NAME, null, DATABASE_VERSION);
        }
        @Override
        public void onCreate(SQLiteDatabase db)
        {
            try{
                db.execSQL(DATABASE_CREATE);
                db.execSQL(DATABASE_CREATE2);
                db.execSQL(DATABASE_CREATE3);
            }catch (SQLException e){
                e.printStackTrace();
            }

        }

        @Override
        public void onUpgrade(SQLiteDatabase db, int oldVersion, int newVersion)
        {
            Log.w(TAG, "Upgrading database from version " + oldVersion + " to " +
                    newVersion + ", which will destroy all old data");
            db.execSQL("DROP TABLE IF EXISTS CarIdentifiers");
            db.execSQL("DROP TABLE IF EXISTS Messages");
            db.execSQL("DROP TABLE IF EXISTS AccidentInformation");
            onCreate(db);
        }
    }
    //opens the database
    public DBAdapter open() throws SQLException
    {
        db = DBHelper.getWritableDatabase();
        return this;
    }

    //closes the database
    public void close(){

        DBHelper.close();
    }
    /*
    *insert user method is used to insert users into the database from the create account page.
    *Each user must have their registration number, password, surname and a uniqueID to be added
    * to the database itself. Content values adds the new values ive gathered from the create
    * account page into an empty set which is then used to add all those values into the
    * Database table selected in the return statement.
     */
    public Long insertUser(String uniqueID, String password, String regNum, String surname){
        ContentValues newValues = new ContentValues();
        newValues.put(KEY_REGISTRATIONNUMBER, regNum);
        newValues.put(KEY_PASSWORD, password);
        newValues.put(KEY_SURNAME, surname);
        newValues.put(KEY_UNIQUEID, uniqueID);
        return db.insert(DATABASE_TABLE, null, newValues);
    }
    /*
    *Insert message is similar to the insert user method but this time it handles the insertion
    * of messages into the database. This is a core element of the system as sending messages
    * from one driver to the other is the whole key to the application. Again here content values
    * creates an empty set for which to store the new values inputted from the activity sendMessage
    * so that they can be inputted in the database table. I added the Date object here so that the
    * messages input a time and a date of when the message was sent so that the user has some
    * reference as to when the incident occurred.
     */
    public Long insertMessage(String message, String messageDate, String recipientReg){
        Date date = new Date(System.currentTimeMillis());
        messageDate = date.toString();
        ContentValues newValues = new ContentValues();
        newValues.put(KEY_MESSAGE, message);
        newValues.put(KEY_DATE, messageDate);
        newValues.put(KEY_REGNO, recipientReg);
        return db.insert(DATABASE_TABLE2, null, newValues);
    }
    /*
    *The insertMarker is again fundamentally the same as insertUser and insertMessage, in the sense
    * that it is just another insert statement into the database. Only this time it is
    * adding a new marker location on a google maps activity referencing a new accident that has
    * occurred. This new accident requires latitude and longitude coordinates to pinpoint the
    * exact location of the accident, also i have opted for a streetname so that users can insert
    * the lcoation of the accident using its name rather than coordinates as that is more user-friendly.
    * The accident time is added automatically with the insertion, which takes the exact system time of
    * the mobile device that inputs the marker.
     */
    public Long insertMarker(String lat, String lng, String streetName, String accidentTime ){
        ContentValues newValues = new ContentValues();
        Date newDate  = new Date(System.currentTimeMillis());
        accidentTime = newDate.toString();
        newValues.put(KEY_LAT, lat);
        newValues.put(KEY_LNG, lng);
        newValues.put(KEY_STREET_NAME, streetName);
        newValues.put(KEY_ACCIDENT_TIME, accidentTime);
        return db.insert(DATABASE_TABLE3, null, newValues);
    }
    /*
    *getSingleEntry method is exactly that. It gets a single entry in the database based on the uniqueID
    * that is fed as the input parameter. This method is called on the homeScreen activity where it checks
    * the uniqueID entered in the textbox against the uniqueIDs stored in the DB.
    * The cursor object here sends an sql query to the database to check if there are in fact any
    * records stored in the DB and if any match the query string input, then retrieve that record.
     */
    public String getSingleEntry(String uniqueID){
        Cursor cursor = db.query(DATABASE_TABLE, null, "UniqueID=?", new String[]{uniqueID}, null, null, null);
        if(cursor.getCount() <1){
            cursor.close();
            return "Entry Doesn't Exist";
        }
        cursor.moveToFirst();
        String returnUniqueID = cursor.getString(cursor.getColumnIndex("UniqueID"));
        cursor.close();
        return returnUniqueID;
    }
    /*
    *getPass method is the same as getsingleEntry only that it retrieves the password.
    *  It gets a single entry in the database based on the password that is fed as the input parameter.
    * This method is called on the homeScreen activity where it checks
    * the password entered in the textbox against the passwords stored in the DB.
    * The cursor object here sends an sql query to the database to check if there are in fact any
    * records stored in the DB and if any match the query string input, then retrieve that record.
     */
    public String getPass(String password) {
        Cursor cursor = db.query(DATABASE_TABLE, null, "Password=?", new String[]{password}, null, null, null);
        if(cursor.getCount() <1){
            cursor.close();
            return "Entry Doesn't Exist";
        }
        cursor.moveToFirst();
        String returnPassword = cursor.getString(cursor.getColumnIndex("Password"));
        cursor.close();
        return returnPassword;
    }
/*
*getAllRows is used on the Inbox form. It takes the registration number taken from the getRegNo
* function and retrieves all the rows that are in the database table for that registration number.
* It gets the id, message, date and the registration number for the regNo entered.
 */
    public Cursor getAllRows(String regNum){
        String where = regNum;
        Cursor c = db.query(DATABASE_TABLE2, new String[] {KEY_ID,KEY_MESSAGE, KEY_DATE, KEY_REGNO},
                KEY_REGNO + "=?", new String[]{(regNum.toUpperCase())}, null, null, null, null);
        if(c !=null){
            c.moveToFirst();
        }
        return c;

    }
/*
*getRegNo is another retrieve sql statement that is used to get the registration number
* of the user who is logged into the system. It takes a uniqueID as input and then looks for the
* registration number in the same column as the uniqueID. Once it finds a match, it pulls the
* registration number that is stored along with the uniqueID entered. Returns the reg number.
 */
    public String getRegNo(String uniqueID) {
        Cursor cursor = db.query(DATABASE_TABLE, new String[]{KEY_REGISTRATIONNUMBER}, KEY_UNIQUEID + "=?",
                new String[]{(uniqueID)},null, null, null, null);
        if(cursor !=null){
            cursor.moveToFirst();
            String returnRegistration = cursor.getString(cursor.getColumnIndex(KEY_REGISTRATIONNUMBER));
            return returnRegistration;
        }
        else{
            return "Sorry no registration numbers exist for that record";
        }
    }

/*
*getMarkers function is used to create a new array list for all the markers in the database table
* it finds the markers information from the table and adds each one to the array list.
* after it adds one marker it moves the cursor object to the next marker if there is one.
 */
    public List<MyMarkerObj> getMarkers(){
        List<MyMarkerObj> markers = new ArrayList<MyMarkerObj>();
        Cursor cursor = db.query(DATABASE_TABLE3, cols, null, null, null, null, null );
        cursor.moveToFirst();
        while(!cursor.isAfterLast()){
            MyMarkerObj m = cursorToMarker(cursor);
            markers.add(m);
            cursor.moveToNext();
        }
        cursor.close();
        return markers;
    }
/*
 * creates a new marker object and uses the setters and getters from the MyMarkerObj class to set
 * each marker object so that it can be stored in the array list in getMarkers function.
 */
    public MyMarkerObj cursorToMarker(Cursor cursor){
        MyMarkerObj m = new MyMarkerObj();
        m.setLat(cursor.getString(1));
        m.setLng(cursor.getString(2));
        m.setStreetName(cursor.getString(3));
        m.setAccidentTime(cursor.getString(4));

        return m;
    }




}
